@@ -1,21 +1,23 @@
 # coding: utf-8
 from IPy import IP
 from os import unlink
-from elixir import *
+from sqlalchemy.types import UnicodeText, Boolean, Integer
+from sqlalchemy.orm import relationship
+from sqlalchemy import Column, ForeignKeyConstraint
+from .elixir import Entity
 from arv.lib.util import valid, try_unique_column, trace, \
         get_keyid_from_certifstring, bin_encoding, \
-        decrypt_privkey, get_keyid_from_certifstring, suppress_colon, \
+        decrypt_privkey, suppress_colon, \
         valid_priv_and_cred, cred_end_validity_date, der_to_pem, \
         split_pkcs7
 from creole import cert
 from arv.db.edge import get_connected_nodes, TmplConnect, Credential, \
-                        get_credential_auth, add_credential_auth, \
-                        add_certificate_authorities
+                        get_credential_auth, add_certificate_authorities
 from arv.lib.logger import logger
 from arv.config import ip_sep
 
 def test_mimetype(mimetype):
-    if mimetype not in (u'ip', u'network'):
+    if mimetype not in ('ip', 'network'):
         raise ValueError('mimetype must be ip or network')
 
 def valid_tmpl_vertex_zephir(zephir_module, zephir_var_ip1, zephir_var_ip2, mimetype):
@@ -24,7 +26,7 @@ def valid_tmpl_vertex_zephir(zephir_module, zephir_var_ip1, zephir_var_ip2, mime
         if zephir_var_ip1 == None:
             raise ValueError('zephir_module and zephir_var_ip1 must be set together')
         zephir_var_ip1 = valid(zephir_var_ip1, 'string')
-        if mimetype == u'ip':
+        if mimetype == 'ip':
             if zephir_var_ip2 != None:
                 raise ValueError('if mimetype is "ip", zephir_var_ip2 must not be set')
         else:
@@ -36,17 +38,14 @@ def valid_tmpl_vertex_zephir(zephir_module, zephir_var_ip1, zephir_var_ip2, mime
     return zephir_module, zephir_var_ip1, zephir_var_ip2
 
 class TmplNode(Entity):
-    # filled at init time
-    name = Field(UnicodeText, unique=True)
-    #(sphynx, etablissement, custom)
-    mimetype = Field(UnicodeText)
-    # filled later
-    vertices = OneToMany('TmplVertex')
-    tmpl_connects_a = OneToMany('TmplConnect', inverse='tmpl_node_a')
-    tmpl_connects_b = OneToMany('TmplConnect', inverse='tmpl_node_b')
-    nodes = OneToMany('Node')
-    allow_master = Field(Boolean, default=True)
-    state = Field(Integer, default=0)
+    __tablename__ = 'arv_db_node_tmplnode'
+    id = Column("id", Integer, primary_key=True)
+    name = Column("name", UnicodeText, unique=True)
+    mimetype = Column("mimetype", UnicodeText)
+    allow_master = Column("allow_master", Boolean, default=True)
+    tmpl_connects_a = relationship('TmplConnect', backref="tmpl_node_a", primaryjoin="TmplConnect.tmpl_node_a_id == TmplNode.id", uselist=True)
+    tmpl_connects_b = relationship('TmplConnect', backref="tmpl_node_b", primaryjoin="TmplConnect.tmpl_node_b_id == TmplNode.id", uselist=True)
+    state = Column('state', Integer, default=0)
 
 
     def add_tmpl_vertex(self, name, mimetype, zephir_module=None, zephir_var_ip1=None, zephir_var_ip2=None):
@@ -84,17 +83,17 @@ def get_tmpl_nodes(mimetype=None):
         raise Exception( 'error in get_tmpl_nodes')
 
 @trace()
-def add_tmpl_node(name, mimetype=u'custom', allow_master=True):
+def add_tmpl_node(name, mimetype='custom', allow_master=True):
     try:
         name = valid(name, 'string')
         mimetype = valid(mimetype, 'string')
         allow_master = valid(allow_master, 'bool')
-        if mimetype not in (u'sphynx', u'etablissement', u'roadwarrior', u'custom'):
+        if mimetype not in ('sphynx', 'etablissement', 'roadwarrior', 'custom'):
             raise ValueError('TmplNode mimetype must be sphynx, etablissement, roadwarrior or custom')
-        if mimetype != u'custom' and TmplNode.query.filter_by(mimetype=mimetype).first() != None:
+        if mimetype != 'custom' and TmplNode.query.filter_by(mimetype=mimetype).first() != None:
             raise ValueError('TmplNode mimetype %s is already used in database' % mimetype)
         return TmplNode(name=name, mimetype=mimetype, allow_master=allow_master)
-    except Exception, e:
+    except Exception as e:
         raise Exception( "error in add_tmpl_node: %s"%str(e))
 
 @trace()
@@ -118,7 +117,7 @@ def create_tmpl_node(name):
     try:
         name = valid(name, 'string')
     except:
-        raise(TypeError, "unsupported encoding in create_node")
+        raise TypeError("unsupported encoding in create_node")
     return Node(name=name)
 
 @trace()
@@ -126,7 +125,7 @@ def get_tmpl_node_by_name(tname):
     try:
         tname = valid(tname, 'string')
         tnode = TmplNode.query.filter_by(name=tname).first()
-    except Exception, e:
+    except Exception as e:
         raise ValueError(str(e))
 
     if tnode == '':
@@ -134,18 +133,19 @@ def get_tmpl_node_by_name(tname):
     return tnode
 
 class TmplVertex(Entity):
-    # filled at init time
-    name = Field(UnicodeText)
-    mimetype = Field(UnicodeText)
-    zephir_module = Field(Integer)
-    zephir_var_ip1 = Field(UnicodeText)
-    zephir_var_ip2 = Field(UnicodeText)
-    tmpl_node = ManyToOne('TmplNode')
-    # filled later
-    tmpl_edges_a = OneToMany('TmplEdge', inverse='tmpl_vertex_a')
-    tmpl_edges_b = OneToMany('TmplEdge', inverse='tmpl_vertex_b')
-    vertices = OneToMany('Vertex')
-    state = Field(Integer, default=0)
+    __tablename__ = 'arv_db_node_tmplvertex'
+    id = Column("id", Integer, primary_key=True)
+    name = Column("name", UnicodeText)
+    mimetype = Column("mimetype", UnicodeText)
+    zephir_module = Column("zephir_module", Integer)
+    zephir_var_ip1 = Column("zephir_var_ip1", UnicodeText)
+    zephir_var_ip2 = Column("zephir_var_ip2", UnicodeText)
+    tmpl_node_id = Column(Integer, index=True)
+    tmpl_node = relationship('TmplNode', backref="vertices", primaryjoin="TmplVertex.tmpl_node_id == TmplNode.id", uselist=False)
+    tmpl_edges_a = relationship('TmplEdge', backref="tmpl_vertex_a", primaryjoin="TmplEdge.tmpl_vertex_a_id == TmplVertex.id", uselist=True)
+    tmpl_edges_b = relationship('TmplEdge', backref="tmpl_vertex_b", primaryjoin="TmplEdge.tmpl_vertex_b_id == TmplVertex.id", uselist=True)
+    state = Column("state", Integer, default=0)
+    __table_args__ = (ForeignKeyConstraint(['tmpl_node_id'], [u'arv_db_node_tmplnode.id'], **{'name': u'arv_db_node_tmplvertex_tmpl_node_id_fk'}),)
 
     def add_vertex(self, node, ip1, ip2=None):
         if node.tmpl_node != self.tmpl_node:
@@ -184,7 +184,7 @@ def create_tmplvertex(name):
         name = valid(name, 'string')
         return TmplVertex(name=name)
     except:
-        raise(ValueError, "unable to create TmplVertex object in internal database")
+        raise ValueError("unable to create TmplVertex object in internal database")
 
 @trace()
 def get_tmpl_vertices(tmplnode=None):
@@ -205,38 +205,47 @@ def get_tmplvertex_by_node(node):
     return templates
 
 # ____________________________________________________________
-
 class Node(Entity):
-    # filled at init time
-    name = Field(UnicodeText, unique=True)
-    uai = Field(UnicodeText)
-    id_zephir = Field(Integer)
-    eole_version = Field(UnicodeText)
-    tmpl_node = ManyToOne('TmplNode')
-    # filled later
-    credentials = OneToMany('Credential')
-    extremities = OneToMany('Extremity')
-    vertices = OneToMany('Vertex')
-    tail_conns = OneToMany('Connect', inverse='head_node')
-    head_conns = OneToMany('Connect', inverse='tail_node')
-    state = Field(Integer, default=0)
+    __tablename__ = 'arv_db_node_node'
+    id = Column("id", Integer, primary_key=True)
+    name = Column("name", UnicodeText, unique=True)
+    uai = Column("uai", UnicodeText)
+    id_zephir = Column("id_zephir", Integer)
+    eole_version = Column("eole_version", UnicodeText)
+    tmpl_node_id = Column(Integer, index=True)
+    tmpl_node = relationship('TmplNode', backref="nodes", primaryjoin="Node.tmpl_node_id == TmplNode.id", uselist=False)
+    state = Column("state", Integer, default=0)
+    credentials = relationship('Credential', backref="node", primaryjoin="Credential.node_id == Node.id", uselist=True)
+    tail_conns = relationship('Connect', backref="head_node", primaryjoin="Connect.head_node_id == Node.id", uselist=True)
+    head_conns = relationship('Connect', backref="tail_node", primaryjoin="Connect.tail_node_id == Node.id", uselist=True)
+    __table_args__ = (ForeignKeyConstraint(['tmpl_node_id'], [u'arv_db_node_tmplnode.id'], **{'name': u'arv_db_node_node_tmpl_node_id_fk'}),)
 
     @trace(hide_args=[6], hide_kwargs=['passwd'])
     def add_credential(self, private_key, credential=None,
             unsigned_credential=None, cred_auth=None, name=None, passwd=None,
            keyid_cred=None, subjkey=None, old_credential=None):
+        if isinstance(credential, str):
+            credential = credential.encode()
+        if isinstance(unsigned_credential, str):
+            unsigned_credential = unsigned_credential.encode()
+        if isinstance(private_key, str):
+            private_key = private_key.encode()
+        if isinstance(keyid_cred, str):
+            keyid_cred = keyid_cred.encode()
+        if isinstance(subjkey, str):
+            subjkey = subjkey.encode()
         subject = cert.get_subject(cert=credential)
-        name = unicode(subject[1], 'utf-8')
+        name = str(subject[1])
         suffix_cred = subject[0]
         suffix_cred = valid(suffix_cred, 'string')
-        subject = bin_encoding("%s/CN=%s"%(suffix_cred, name))
+        subject = bin_encoding("{0}, CN = {1}".format(suffix_cred, name))
         expiration_date = cred_end_validity_date(credential)
 
         # Test du modulus
         if not valid_priv_and_cred(private_key, credential, passwd):
             raise ValueError("Credential and private_key doesn't match")
         # On ne déchiffre que la clé privée du node Sphynx-ARV
-        tmpl_nodes = get_tmpl_nodes(u'sphynx')
+        tmpl_nodes = get_tmpl_nodes('sphynx')
         current_node = tmpl_nodes[0].nodes[0]
         if self == current_node:
             private_key = decrypt_privkey(privkey_string=private_key, passwd=passwd)
@@ -262,21 +271,22 @@ class Node(Entity):
             old_credential=None, old_ca=None):
         ca = None
         cert_type, certificate = der_to_pem(credential)
-        if old_ca is None:
-            ca = add_certificate_authorities(certificate, cert_type)
         if cert_type == "pkcs7":
-            ca_cred, certificate = split_pkcs7(certificate)
+            ca_chain, certificate = split_pkcs7(certificate)
+            if old_ca is None:
+                for ca_cred in ca_chain:
+                    ca = add_certificate_authorities(ca_cred, 'x509')
         if certificate is not None:
             try:
                 cred_subject = cert.get_issuer_subject(cert=certificate)
-                name = unicode(cred_subject[1], 'utf-8')
+                name = cred_subject[1]
                 suffix_cred = cred_subject[0]
                 suffix_cred = valid(suffix_cred, 'string')
-                cred_subject = bin_encoding("%s/CN=%s"%(suffix_cred, name))
+                cred_subject = bin_encoding("{0}, CN = {1}".format(suffix_cred, name))
                 ca = Credential.query.filter_by(subject=cred_subject).first()
                 if ca == None:
                     raise Exception("CA does not exists in database, Can't add certificate")
-            except Exception, e:
+            except Exception as e:
                 raise ValueError(str(e))
         else:
             raise Exception("Unknown certificate type")
@@ -301,7 +311,7 @@ class Node(Entity):
             else:
                 priv_ip = pub_ip
             return Extremity(pub_ip=pub_ip, priv_ip=priv_ip, node=self)
-        except Exception, e:
+        except Exception as e:
             raise ValueError(str(e))
 
     def get_credentials(self, cred_auth=None):
@@ -311,23 +321,20 @@ class Node(Entity):
             else:
                 return Credential.query.filter_by(node=self,
                         cred_auth=cred_auth).all()
-        except Exception, e:
+        except Exception as e:
             raise ValueError(str(e))
 
     def get_extremities(self):
         try:
             return Extremity.query.filter_by(node=self).all()
-        except Exception, e:
+        except Exception as e:
             raise ValueError(str(e))
 
 @trace()
 def get_etabs():
     ret = []
     for node in get_nodes():
-        if type(node.name) == unicode:
-            libelle = unicode.encode(node.name, "utf-8")
-        else:
-            libelle = node.name
+        libelle = node.name
         ret.append((libelle, node.uai, node.id_zephir, node.eole_version, node.tmpl_node.mimetype))
     return ret
 
@@ -369,16 +376,23 @@ def del_node(node):
             credential.delete()
 
         node.delete()
-    except Exception, e:
+    except Exception as e:
         raise Exception("error in del_node" % str(e))
 
+
 class Vertex(Entity):
-    # filled at init time
-    ip1 = Field(UnicodeText)
-    ip2 = Field(UnicodeText)
-    tmpl_vertex = ManyToOne('TmplVertex')
-    node = ManyToOne('Node')
-    state = Field(Integer, default=0)
+    __tablename__ = 'arv_db_node_vertex'
+    id = Column("id", Integer, primary_key=True)
+    ip1 = Column("ip1", UnicodeText)
+    ip2 = Column("ip2", UnicodeText)
+    tmpl_vertex_id = Column(Integer, index=True)
+    tmpl_vertex = relationship('TmplVertex', backref="vertices", primaryjoin="Vertex.tmpl_vertex_id == TmplVertex.id", uselist=False)
+    node_id = Column(Integer, index=True)
+    node = relationship('Node', backref="vertices", primaryjoin="Vertex.node_id == Node.id", uselist=False)
+    state = Column("state", Integer, default=0)
+    __table_args__ = (ForeignKeyConstraint(['tmpl_vertex_id'], [u'arv_db_node_tmplvertex.id'], **{'name': u'arv_db_node_vertex_tmpl_vertex_id_fk'}),
+                      ForeignKeyConstraint(['node_id'], [u'arv_db_node_node.id'], **{'name': u'arv_db_node_vertex_node_id_fk'})
+    )
 
     @trace()
     def set_ips(self, ip1, ip2):
@@ -387,17 +401,17 @@ class Vertex(Entity):
         for ip1_ in ip1.split(ip_sep):
             vip1.append(valid(ip1_, 'ip'))
         ip1 = ip_sep.join(vip1)
-        if self.tmpl_vertex.mimetype != u'ip':
+        if self.tmpl_vertex.mimetype != 'ip':
             for ip2_ in ip2.split(ip_sep):
                 vip2.append(valid(ip2_, 'ip'))
             if len(vip1) != len(vip2):
                 raise Exception('Error: length of ip1 different than ip2')
             ip2 = ip_sep.join(vip2)
 
-        if self.tmpl_vertex.mimetype == u'ip':
+        if self.tmpl_vertex.mimetype == 'ip':
             if ip2 != None:
                 raise ValueError('No ip2 if mimetype is ip')
-        elif self.tmpl_vertex.mimetype == u'network':
+        elif self.tmpl_vertex.mimetype == 'network':
             try:
                 for i in range(0, len(vip1)):
                     IP("%s/%s"%(vip1[i], vip2[i]))
@@ -421,15 +435,19 @@ def get_vertex_by_node(node):
 def get_vertex_by_node_tmplvertex(node, tmpl_vertex):
     return Vertex.query.filter_by(node=node, tmpl_vertex=tmpl_vertex).first()
 
-class Extremity(Entity): # connection ip
-    # filled at init time
-    pub_ip = Field(UnicodeText, unique=True)
-    priv_ip = Field(UnicodeText)
-    node = ManyToOne('Node')
-    # filled later
-    head_conns = OneToMany('Connect', inverse='head_extr')
-    tail_conns = OneToMany('Connect', inverse='tail_extr')
-    state = Field(Integer, default=0)
+
+class Extremity(Entity):
+    __tablename__ = 'arv_db_node_extremity'
+    id = Column("id", Integer, primary_key=True)
+    pub_ip = Column("pub_ip", UnicodeText, unique=True)
+    priv_ip = Column("priv_ip", UnicodeText)
+    node_id = Column(Integer, index=True)
+    state = Column("state", Integer, default=0)
+    node = relationship('Node', backref="extremities", primaryjoin="Extremity.node_id == Node.id", uselist=False)
+    head_conns = relationship('Connect', backref="head_extr", primaryjoin="Connect.head_extr_id == Extremity.id", uselist=True)
+    tail_conns = relationship('Connect', backref="tail_extr", primaryjoin="Connect.tail_extr_id == Extremity.id", uselist=True)
+    __table_args__ = (ForeignKeyConstraint(['node_id'], [u'arv_db_node_node.id'], **{'name': u'arv_db_node_extremity_node_id_fk'}),)
+
 
 @trace()
 def del_extremity(extremity):
