@@ -7,7 +7,7 @@ import binascii
 from tempfile import mkstemp
 from os import unlink, system, sep, listdir, close, open as osopen, fdopen, O_WRONLY, O_CREAT
 from os.path import basename, dirname, isdir, join, exists
-from commands import getoutput, getstatusoutput
+from subprocess import getoutput, getstatusoutput
 from pyeole.process import system_out
 import subprocess
 
@@ -60,11 +60,8 @@ def trace(hide_args=None, hide_kwargs=None):
 
 @trace()
 def normalize_unicode(string):
-    if not type(string) == unicode:
-        if type(string) == str:
-            string = unicode(string, "utf-8")
-        else:
-            raise TypeError( "unsupported encoding")
+    if not isinstance(string, str):
+        raise TypeError( "unsupported encoding")
     return string
 
 @trace()
@@ -72,6 +69,8 @@ def valid(value, typ):
     """
         formencode validation
     """
+    if isinstance(value, bytes):
+        value = value.decode()
     if typ == 'string':
         validator = validators.String()
     if typ == 'bool':
@@ -93,12 +92,12 @@ def valid(value, typ):
 def try_unique_column(func_name, function, **args):
     try:
         return function(**args)
-    except IntegrityError, e:
+    except IntegrityError as e:
         if " column name is not unique " in str(e):
             raise Exception("Name should be unique")
         else:
             raise Exception("error in %s: %s" % (func_name, str(e)))
-    except Exception, e:
+    except Exception as e:
         raise Exception("error in %s: %s" % (func_name, str(e)))
 
 # ____________________________________________________________
@@ -148,20 +147,23 @@ def get_lastline(lines):
 def bin_encoding(clear):
     """encode in binary for strongswan database
     """
+    clear = clear.encode()
     # WARNING : THIS DOESN'T REPLACE ID2SQL SCRIPT.
     # THIS IS JUST TO MAKE ARV DATABASE BUILDNING POSSIBLE WITHOUT ID2SQL
     # STRONGSWAN DATABASE MODE SHOULDN'T WORKS WITH THAT
     result = binascii.hexlify(clear)
     #encode and decode. this functionality already exists with the encodings library (which is built-in)
     # have to decode to put the normal string in a sqlalchemy binary type
-    logger.debug("#-> bin_encoding result var: " + result)
-    return result.decode("hex")
+    logger.debug("#-> bin_encoding result var: " + result.decode())
+    return result
 
 @trace()
 def suppress_colon(keyid):
     """suppress colons in a string
     """
-    return keyid.replace(':', '').decode('hex')
+    if isinstance(keyid, bytes):
+        keyid = keyid.decode()
+    return bytes.fromhex(keyid.replace(':', ''))
 
 @trace(hide_args=[1], hide_kwargs=['passwd'])
 def get_keyid_in_certif(certif_name, passwd=None, certiftype='ca'):
@@ -176,10 +178,10 @@ def get_keyid_in_certif(certif_name, passwd=None, certiftype='ca'):
         priv_certname = cert.ca_keyfile
     else:
         priv_certname = join(certif_path, 'priv-'+certif_name)
-    return get_keyid_from_keyid_in_certif(priv_certname, passwd, mode='rsa')
+    return get_keyid_from_keyid_in_certif(priv_certname, passwd, mode='rsa', ca=certiftype=='ca')
 
 @trace(hide_args=[1], hide_kwargs=['passwd'])
-def get_keyid_from_keyid_in_certif(certif_name, passwd=None, mode='rsa'):
+def get_keyid_from_keyid_in_certif(certif_name, passwd=None, mode='rsa', ca=False):
     """constructs a keyid when a keyid is not present in the certificate
     """
     if mode == 'rsa':
@@ -190,10 +192,10 @@ def get_keyid_from_keyid_in_certif(certif_name, passwd=None, mode='rsa'):
                        '-passin', 'stdin', '-outform', 'DER']
 
             pub_info_cmd = rsa_cmd + ['-pubout']
-            code, pub_info_der, stderr = system_out(pub_info_cmd, stdin=passwd)
+            code, pub_info_der, stderr = system_out(pub_info_cmd, stdin=passwd, to_str=False)
 
             pub_key_cmd = rsa_cmd + ['-RSAPublicKey_out']
-            code, pub_key_der, stderr = system_out(pub_key_cmd, stdin=passwd)
+            code, pub_key_der, stderr = system_out(pub_key_cmd, stdin=passwd, to_str=False)
 
             code, pub_info_fpr, stderr = system_out(sha1_cmd, pub_info_der)
             code, pub_key_fpr, stderr = system_out(sha1_cmd, pub_key_der)
@@ -205,27 +207,33 @@ def get_keyid_from_keyid_in_certif(certif_name, passwd=None, mode='rsa'):
             raise Exception('Invalid password')
     elif mode == 'x509':
         #Extract from credential
-        cmd = 'openssl {0} -in "{1}" -text -noout| grep keyid'.format(mode, certif_name)
+        if ca:
+            ext = 'subjectKeyIdentifier'
+        else:
+            ext = 'authorityKeyIdentifier'
+        cmd = 'openssl {0} -in "{1}" -noout -ext {2} | tail -n 1'.format(mode, certif_name, ext)
         output = getoutput(cmd)
-        keyid = output.split("keyid:")[1]
+        keyid = output.strip()
         logger.debug('{0}'.format(keyid))
         subjkey = keyid
     return (subjkey, keyid)
 
 @trace(hide_args=[1], hide_kwargs=['passwd'])
-def get_keyid_from_certifstring(certif_string, passwd=None, mode='rsa'):
+def get_keyid_from_certifstring(certif_string, passwd=None, mode='rsa', ca=False):
     """constructs a keyid when a keyid is not present in the certificate
     """
     try:
         fd, certif_name = mkstemp()
-        fh = file(certif_name, 'w')
+        fh = open(certif_name, 'wb')
+        if isinstance(certif_string, str):
+            certif_string = certif_string.encode()
         fh.write(certif_string)
         fh.close()
-        subjkey, keyid = get_keyid_from_keyid_in_certif(certif_name, passwd, mode=mode)
+        subjkey, keyid = get_keyid_from_keyid_in_certif(certif_name, passwd, mode=mode, ca=ca)
         close(fd)
         unlink(certif_name)
         return (subjkey, keyid)
-    except Exception, e:
+    except Exception as e:
         msg = 'Cannot generate keyid in certificate : %s'% str(e)
         logger.warning(msg)
         raise Exception(msg)
@@ -234,11 +242,16 @@ def get_keyid_from_certifstring(certif_string, passwd=None, mode='rsa'):
 def password_OK(private_key, password):
     """Test private_key password validity
     """
+    if isinstance(password, str):
+        password = password.encode()
     if password:
         cmd = ["openssl", "rsa",  "-in", private_key, "-passin", "stdin"]
     else:
         cmd = ["openssl", "rsa",  "-in", private_key]
-    logger.debug("PASSWORD : {0}".format(password))
+    if password:
+        logger.debug("PASSWORD : {0}".format(password.decode()))
+    else:
+        logger.debug("PAS DE PASSWORD")
     process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
     if password:
         process.communicate(input=password)
@@ -253,10 +266,14 @@ def decrypt_privkey(privkey_string, passwd):
     """Suppress password from private key
     """
     fd, privkey_filename = mkstemp()
-    fh = file(privkey_filename, 'w')
+    fh = open(privkey_filename, 'wb')
+    if isinstance(privkey_string, str):
+        privkey_string = privkey_string.encode()
     fh.write(privkey_string)
     fh.close()
     close(fd)
+    if isinstance(passwd, str):
+        passwd = passwd.encode()
     cmd = ["openssl", "rsa",  "-in", privkey_filename, "-passin", "stdin"]
     process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
     openssl_output = process.communicate(input=passwd)[0]
@@ -264,9 +281,10 @@ def decrypt_privkey(privkey_string, passwd):
     unlink(privkey_filename)
     if retcode:
         raise Exception('Unable to decrypt private key, check password')
-    lines = openssl_output.split('\n')
-    decrypted_key = "\n".join(lines)
-    return decrypted_key
+    #lines = openssl_output.decode().split('\n')
+    #decrypted_key = "\n".join(lines)
+    #return decrypted_key
+    return openssl_output
 
 def get_req_archive(name):
     """Download credential request to send to CA
@@ -282,7 +300,7 @@ tar -czf {3}.tgz {4} {5}
 rm {4} {5}
 """.format(ssl_dir, reqfilename, privkeyfilename, name, reqname, privkeyname))
     tarfname = join(ssl_dir, name+".tgz")
-    fh = file(tarfname, 'r')
+    fh = open(tarfname, 'rb')
     content = fh.read()
     fh.close()
     unlink(tarfname)
@@ -292,8 +310,10 @@ rm {4} {5}
 def gen_archive_name(uai, name):
     """
     """
+    if isinstance(name, bytes):
+        name = name.decode()
     amonpath = vpn_path + sep + str(uai)
-    archivename = str(uai) + "-" + unicode.encode(name, "utf-8") + ".tar.gz"
+    archivename = str(uai) + "-" + name + ".tar.gz"
     return amonpath, archivename
 
 @trace()
@@ -302,23 +322,18 @@ def der_to_pem(certificate):
     Convert certificate to PEM format
     Returns tuple : (cert_type, pem_cert)
     """
-    fd, certif_filename = mkstemp()
-    fh = file(certif_filename, 'w')
-    fh.write(certificate)
-    fh.close()
-    close(fd)
+    if isinstance(certificate, str):
+        certificate = certificate.encode()
     cert_types = ["x509", "pkcs7"]
     cert_formats = ["DER", "PEM"]
     for cert_type in cert_types:
         for cert_format in cert_formats:
-            cmd = ["openssl", cert_type, "-inform", cert_format, "-in", certif_filename]
-            process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
-            ret_code = process.wait()
-            if ret_code == 0:
-                unlink(certif_filename)
-                return cert_type, process.stdout.read()
-    unlink(certif_filename)
-    return None, None
+            cmd = ["openssl", cert_type, "-inform", cert_format]
+            with subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False) as process:
+                output, err = process.communicate(input=certificate)
+                if process.returncode == 0:
+                    return cert_type, output
+    raise Exception(f'unable to convert certificat in PEM format: {output}, {err}')
 
 @trace()
 def is_ca(certificate):
@@ -326,17 +341,62 @@ def is_ca(certificate):
     certificate is in PEM format
     Returns True if certificate is a CA, False if not
     """
+    if isinstance(certificate, str):
+        certificate = certificate.encode()
     cmd = ["openssl", "x509", "-noout", "-text"]
-    process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
-    process.stdin.write(certificate)
-    ret_code = process.wait()
-    if ret_code != 0:
-        raise Exception("Error in execution 'openssl x509' command")
-    text_cert = process.stdout.read().splitlines()
-    for line in text_cert:
-        if re.match(r'(.*)CA:TRUE(.*)', line):
-            return True
-    return False
+    with subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False) as process:
+        decoded_certificate, err = process.communicate(input=certificate)
+        if process.returncode !=0:
+            raise Exception(f"Error in execution 'openssl x509' command: {err}")
+    match =  re.match(rb'.*CA:TRUE.*', decoded_certificate, re.S)
+    return match != None
+
+class CertChain:
+    def __init__(self):
+        self.signed_by = {}
+        self.certs = {}
+
+    def add_cert(self, certif):
+        cert_subject = cert.get_subject(certif)[1]
+        if cert_subject not in self.certs:
+            self.certs[cert_subject] = certif
+            cert_issuer = cert.get_issuer_subject(certif)[1]
+            self.signed_by[cert_subject] = cert_issuer
+        
+    def add_certs(self, certifs):
+        for certif in certifs:
+            self.add_cert(certif)
+
+    def get_chain_by_subject(self, leaf_certif_subject, subject_only=False):
+        if not leaf_certif_subject in self.certs:
+            raise Exception('Unknown certificate')
+        chain = [leaf_certif_subject]
+        subject = leaf_certif_subject
+
+        while self.signed_by.get(subject, None):
+            issuer = self.signed_by[subject]
+            if subject == issuer:
+                break
+            if self.certs.get(issuer, None):
+                chain.append(self.signed_by[subject])
+            else:
+                break
+            subject = issuer
+        chain.reverse()
+        if not subject_only:
+            chain = [self.certs[subject] for subject in chain]
+        return chain
+
+    def get_chain_by_b64(self, leaf_certif, subject_only=False):
+        subject = cert.get_subject(certif)[1]
+        return self.get_chain_by_subject(subject, subject_only=subject_only)
+
+    def get_leaf_certificate(self):
+        leaf_certificates = [c for c in self.certs if c not in self.signed_by.values()]
+        if len(leaf_certificates) != 1:
+            raise Exception('Ambiguous or gapped certificate chain')
+        return leaf_certificates[0]
+
 
 @trace()
 def split_pkcs7(pkcs7_cred):
@@ -348,61 +408,32 @@ def split_pkcs7(pkcs7_cred):
     """
     try:
         cmd = ["openssl", "pkcs7", "-print_certs"]
-        process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
-        process.stdin.write(pkcs7_cred)
-        ret_code = process.wait()
-        if ret_code != 0:
-            raise Exception("Error in execution 'openssl pkcs7' command")
-        creds_chain = process.stdout.read()
+        if isinstance(pkcs7_cred, str):
+            pkcs7_cred = pkcs7_cred.encode()
+        if not pkcs7_cred.startswith(b'-----BEGIN PKCS7-----'):
+            cmd.extend(['-inform', 'der'])
+        with subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False) as process:
+            creds_chain, err = process.communicate(input=pkcs7_cred)
+            if process.returncode != 0:
+                raise Exception(f"Error in execution 'openssl pkcs7' command: {err}")
         # Append cert_list
-        append_cert_line = False
-        new_cert = True
-        cert_list = []
-        index = 0
-        for line in creds_chain.splitlines():
-            if line == '-----BEGIN CERTIFICATE-----':
-                append_cert_line = True
-                new_cert = True
-            if append_cert_line:
-                if new_cert:
-                    cert_list.append('{0}\n'.format(line))
-                    new_cert = False
-                else:
-                    cert_list[index] += '{0}\n'.format(line)
-            if line == '-----END CERTIFICATE-----':
-                append_cert_line = False
-                index +=1
-        # Filter certificates :
-        #  - CAs in ca_certs dict
-        #  - end-user cerificate in cred
-        ca_certs= {}
-        cred = ''
-        for certif in cert_list:
-            if is_ca(certif):
-                cacert_subject = cert.get_subject(certif)[1]
-                if cacert_subject not in ca_certs:
-                    ca_certs[cacert_subject] = certif
-            else:
-                cred = certif
-        # Get root ca certificate subject
-        ca_subject = None
-        issuer_ca_subject = None
-        for ca_subject, ca_cert in ca_certs.items():
-            issuer_ca_subject = cert.get_issuer_subject(ca_cert)[1]
-            if ca_subject == issuer_ca_subject:
-                break
-        # Sort CA certificates
-        ca_creds = []
-        while True and ca_certs != {}:
-            ca_creds.append(ca_certs.get(ca_subject))
-            issuer_ca_subject = ca_subject
-            ca_subject = next((ca_subj for ca_subj, ca_cert in ca_certs.items() if cert.get_issuer_subject(ca_cert)[1] == issuer_ca_subject and cert.get_subject(ca_cert)[1] != issuer_ca_subject), None)
-            if ca_subject == None:
-                break
-        return (ca_creds, cred)
-    except Exception, e:
-        logger.error('Cannot read pkcs7: {1}'.format(e))
-        raise Exception('Cannot read pkcs7: {1}'.format(e))
+        pem_cert_re = re.compile(r'^(-----BEGIN CERTIFICATE-----.*?-----END CERTIFICATE-----)', re.M|re.S)
+        cert_list = pem_cert_re.findall(creds_chain.decode())
+        cert_chain = CertChain()
+        cert_chain.add_certs(cert_list)
+        chain = cert_chain.get_chain_by_subject(cert_chain.get_leaf_certificate())
+        if is_ca(chain[-1]):
+            ca_chain = chain
+            machine_certificate = None
+        else:
+            ca_chain = chain[:-1]
+            machine_certificate = chain[-1]
+        return ca_chain, machine_certificate
+    except UnicodeDecodeError as e:
+        raise Exception("Problème de lecture du fichier : %s" % e)
+    except Exception as e:
+        logger.error('Cannot read pkcs7: {0}'.format(e))
+        raise Exception('Cannot read pkcs7: {0}'.format(e))
 
 @trace(hide_args=[1, 2], hide_kwargs=['pkcs12_password', 'key_passphrase'])
 def split_pkcs12(pkcs12_certificate, pkcs12_password, key_passphrase):
@@ -412,13 +443,17 @@ def split_pkcs12(pkcs12_certificate, pkcs12_password, key_passphrase):
         pkcs12_pwd_file = "/root/p12_pwd"
         if exists(pkcs12_pwd_file):
             unlink(pkcs12_pwd_file)
-        fh = fdopen(osopen(pkcs12_pwd_file, O_WRONLY|O_CREAT, 0400), 'w')
+        fh = fdopen(osopen(pkcs12_pwd_file, O_WRONLY|O_CREAT, 0o400), 'w')
+        if isinstance(pkcs12_password, bytes):
+            pkcs12_password = pkcs12_password.decode()
         fh.write(pkcs12_password)
         fh.close()
         key_pwd_file = "/root/key_pwd"
         if exists(key_pwd_file):
             unlink(key_pwd_file)
-        fh = fdopen(osopen(key_pwd_file, O_WRONLY|O_CREAT, 0400), 'w')
+        fh = fdopen(osopen(key_pwd_file, O_WRONLY|O_CREAT, 0o400), 'w')
+        if isinstance(key_passphrase, bytes):
+            key_passphrase = key_passphrase.decode()
         fh.write(key_passphrase)
         fh.close()
         cmd = 'openssl pkcs12 -passin file:{0} -passout file:{1}'.format(pkcs12_pwd_file, key_pwd_file).split()
@@ -428,6 +463,8 @@ def split_pkcs12(pkcs12_certificate, pkcs12_password, key_passphrase):
         if ret_code != 0:
             raise Exception("Error in execution 'openssl pkcs12' command")
         pkcs12_chain = process.stdout.read()
+        if isinstance(pkcs12_chain, bytes):
+            pkcs12_chain = pkcs12_chain.decode()
         logger.debug('PKCS12 command result' + pkcs12_chain)
         unlink(pkcs12_pwd_file)
         unlink(key_pwd_file)
@@ -439,7 +476,9 @@ def split_pkcs12(pkcs12_certificate, pkcs12_password, key_passphrase):
         certs_chain = '\n'.join(certs_pattern.findall(pkcs12_chain))
         # generate pkcs7 certificate
         fd, tmp_certs_chain_file = mkstemp()
-        fh = file(tmp_certs_chain_file, 'w')
+        fh = open(tmp_certs_chain_file, 'w')
+        if isinstance(certs_chain, bytes):
+            certs_chain = certs_chain.decode()
         fh.write(certs_chain)
         fh.close()
         cmd = 'openssl crl2pkcs7 -nocrl -certfile {0}'.format(tmp_certs_chain_file).split()
@@ -450,7 +489,7 @@ def split_pkcs12(pkcs12_certificate, pkcs12_password, key_passphrase):
         pkcs7_certs = process.stdout.read()
         unlink(tmp_certs_chain_file)
         return private_key, pkcs7_certs
-    except Exception, e:
+    except Exception as e:
         raise Exception('Cannot read pkcs12: {0}'.format(e))
 
 @trace()
@@ -493,12 +532,31 @@ def ipsec_running():
     else:
         return False
 
+@trace()
+def ipsec_stop():
+    """
+    """
+    ipsec_stop = "service strongswan-starter stop > /dev/null"
+    errcode, output = getstatusoutput(ipsec_stop)
+    return errcode
+
+@trace()
+def ipsec_start():
+    """
+    """
+    ipsec_start = "service strongswan-starter start > /dev/null"
+    errcode, output = getstatusoutput(ipsec_start)
+    return errcode
+
 @trace()
 def ipsec_restart():
     """
     """
-    ipsec_restart = "service strongswan restart > /dev/null"
-    errcode, output = getstatusoutput(ipsec_restart)
+    errcode = ipsec_stop()
+    if errcode != 0:
+        return errcode
+    errcode = ipsec_start()
+    return errcode
 
 @trace()
 def ipsec_down(connstring):
@@ -531,6 +589,8 @@ def fill_file(from_filename, to_filename):
     content = from_fd.read()
     purge_file(to_filename)
     to_fd = open(to_filename, "w")
+    if isinstance(content, bytes):
+        content = content.decode()
     to_fd.write(content)
     from_fd.close()
     to_fd.close()
@@ -559,10 +619,12 @@ def valid_priv_and_cred(private_key, credential, passwd):
     """Test private key and credential compatibility
     """
     fd, privkey_filename = mkstemp()
-    fh = file(privkey_filename, 'w')
+    fh = open(privkey_filename, 'wb')
     fh.write(private_key)
     fh.close()
     close(fd)
+    if isinstance(passwd, str):
+        passwd = passwd.encode()
     cmd = ["openssl", "rsa",  "-in", privkey_filename, "-noout", "-modulus", "-passin", "stdin"]
     process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
     privkey_modulus = process.communicate(input=passwd)[0]
@@ -570,7 +632,9 @@ def valid_priv_and_cred(private_key, credential, passwd):
     unlink(privkey_filename)
     if retcode == 0:
         fd, certif_filename = mkstemp()
-        fh = file(certif_filename, 'w')
+        fh = open(certif_filename, 'wb')
+        if isinstance(credential, str):
+            credential = credential.encode()
         fh.write(credential)
         fh.close()
         close(fd)
@@ -589,7 +653,7 @@ def cred_end_validity_date(credential):
         from "Mar 22 09:32:39 2015 GMT" format
         to "22/03/2015" format
     """
-    cred_cmd = 'echo "{0}" | openssl x509 -noout -dates|grep ^"notAfter="'.format(credential)
+    cred_cmd = 'echo "{0}" | openssl x509 -noout -dates|grep ^"notAfter="'.format(credential.decode())
     errcode, not_after_date = getstatusoutput(cred_cmd)
     if errcode == 0:
         try:
@@ -603,6 +667,8 @@ def cred_end_validity_date(credential):
 def escape_special_characters(text, characters='\\\'"'):
     """Escape special characters
     """
+    if isinstance(text, bytes):
+        text = text.decode()
     for character in characters:
         text = text.replace( character, '\\' + character )
     return text
@@ -621,6 +687,6 @@ def remove_special_characters(text, characters='\\\'"'):
     """Remove special characters
     """
     for character in characters:
-        text = text.replace( character, '' )
+        text = text.replace(character, '')
     return text
 
