@@ -64,7 +64,9 @@ ARV.observable = (function() {
             ARV.addnode1.openWindow(
                 selectid,
                 ARV.viewcred.getSelectedUAI(),
-                ARV.viewcred.getSelectedName()
+                ARV.viewcred.getSelectedName(),
+                ARV.viewcred.getSelectedIdZephir(),
+                ARV.viewcred.getSelectedMimetype()
             )
         },
         //treenode 2
